# -*- coding: utf-8 -*-
import logging
from logging import StreamHandler
from typing import Any, Dict, Union

from flask import Blueprint, Flask, after_this_request, request, Response
from flask_caching import Cache

from chaosplt_experiment.auth import setup_jwt, setup_login
from chaosplt_experiment.schemas import setup_schemas
from chaosplt_experiment.service import Services
from chaosplt_experiment.storage import ExperimentStorage, ExecutionStorage

from .execution import api as execution_api
from .experiment import api as experiment_api

__all__ = ["create_api", "cleanup_api", "serve_experiment_api",
           "serve_execution_api"]


def create_api(config: Dict[str, Any]) -> Flask:
    app = Flask(__name__)

    app.url_map.strict_slashes = False
    app.debug = config.get("debug", False)

    logger = logging.getLogger('flask.app')
    logger.propagate = False

    app.config["SECRET_KEY"] = config["http"]["secret_key"]
    app.secret_key = config["http"]["secret_key"]
    app.config["JWT_SECRET_KEY"] = config["jwt"]["secret_key"]
    app.config["SQLALCHEMY_DATABASE_URI"] = config["db"]["uri"]

    app.config["CACHE_TYPE"] = config["cache"].get("type", "simple")
    if app.config["CACHE_TYPE"] == "redis":
        redis_config = config["cache"]["redis"]
        app.config["CACHE_REDIS_HOST"] = redis_config.get("host")
        app.config["CACHE_REDIS_PORT"] = redis_config.get("port", 6379)
        app.config["CACHE_REDIS_DB"] = redis_config.get("db", 0)
        app.config["CACHE_REDIS_PASSWORD"] = redis_config.get("password")

    setup_jwt(app)
    setup_schemas(app)
    setup_login(app, from_jwt=True)

    return app


def cleanup_api(app: Flask):
    pass


def serve_experiment_api(app: Flask, cache: Cache, services: Services,
                         storage: ExperimentStorage, config: Dict[str, Any],
                         mount_point: str = '/api/v1/experiments',
                         log_handler: StreamHandler = None):
    register_experiment_api(app, cache, services, storage)


def serve_execution_api(app: Flask, cache: Cache, services: Services,
                        storage: ExecutionStorage, config: Dict[str, Any],
                        mount_point: str = '/api/v1/executions',
                        log_handler: StreamHandler = None):
    register_execution_api(app, cache, services, storage)


###############################################################################
# Internals
###############################################################################
def register_experiment_api(app: Flask, cache: Cache, services: Services,
                            storage: ExperimentStorage):
    patch_request(experiment_api, services, storage)
    app.register_blueprint(experiment_api, url_prefix="/experiments")


def register_execution_api(app: Flask, cache: Cache, services: Services,
                           storage: ExecutionStorage):
    patch_request(execution_api, services, storage)
    app.register_blueprint(execution_api, url_prefix="/executions")


def patch_request(bp: Blueprint, services,
                  storage: Union[ExperimentStorage, ExecutionStorage]):
    @bp.before_request
    def prepare_request():
        request.services = services
        request.storage = storage

        @after_this_request
        def clean_request(response: Response):
            request.services = None
            request.storage = None
            return response



# -*- coding: utf-8 -*-
import cherrypy
import toml

__all__ = ["load_settings"]


def load_settings(toml_path: str):
    """
    Load settings from the environment.
    """
    config = toml.load(toml_path)
    if "chaosplatform" in config:
        config = config["chaosplatform"]

    debug = config.get("debug", False)
    server_addr = config["http"]["address"]
    host, port = server_addr.rsplit(":", 1)
    default_cherrypy_env = "" if debug else "production"

    cherrypy_config = config["http"].get("cherrypy", {})
    cherrypy.engine.unsubscribe('graceful', cherrypy.log.reopen_files)
    cherrypy.config.update({
        'server.socket_host': server_addr,
        'server.socket_port': server_addr,
        'engine.autoreload.on': False,
        'log.screen': debug,
        'log.access_file': cherrypy_config.get("access_file", ""),
        'log.error_file': cherrypy_config.get("error_file", ""),
        'environment': cherrypy_config.get(
            "environment", default_cherrypy_env)
    })

    if "proxy" in config["http"]:
        cherrypy.config.update({
            'tools.proxy.on': True,
            'tools.proxy.base': config["http"]["proxy"]
        })

    return config



import click

from . import __version__
from .log import configure_logger
from .server import run_forever
from .settings import load_settings


@click.group()
@click.version_option(version=__version__)
def cli():
    pass


@cli.command()
@click.option('--config',
              type=click.Path(exists=True, readable=True, resolve_path=True),
              help='Configuration TOML file.')
@click.option('--logger-config',
              type=click.Path(exists=False, readable=True, resolve_path=True),
              help='Python logger JSON definition.')
def run(config: str = None, logger_config: str = None):
    """
    Runs the application.
    """
    config = load_settings(config)
    configure_logger(logger_config, config)
    run_forever(config)
